@@ -93,6 +93,7 @@ main (gint   argc,
   gboolean force = FALSE;
   gboolean use_trace_fd = FALSE;
   gboolean gnome_shell = FALSE;
+  gboolean rapl = FALSE;
   int pid = -1;
   int fd;
   int flags;
@@ -100,7 +101,7 @@ main (gint   argc,
     { "pid", 'p', 0, G_OPTION_ARG_INT, &pid, N_("Make sysprof specific to a task"), N_("PID") },
     { "command", 'c', 0, G_OPTION_ARG_STRING, &command, N_("Run a command and profile the process"), N_("COMMAND") },
     { "force", 'f', 0, G_OPTION_ARG_NONE, &force, N_("Force overwrite the capture file") },
-    { "no-battery", 0, 0, G_OPTION_ARG_NONE, &no_cpu, N_("Disable recording of battery statistics") },
+    { "no-battery", 0, 0, G_OPTION_ARG_NONE, &no_battery, N_("Disable recording of battery statistics") },
     { "no-cpu", 0, 0, G_OPTION_ARG_NONE, &no_cpu, N_("Disable recording of CPU statistics") },
     { "no-disk", 0, 0, G_OPTION_ARG_NONE, &no_disk, N_("Disable recording of Disk statistics") },
     { "no-perf", 0, 0, G_OPTION_ARG_NONE, &no_perf, N_("Do not record stacktraces using Linux perf") },
@@ -110,6 +111,7 @@ main (gint   argc,
     { "use-trace-fd", 0, 0, G_OPTION_ARG_NONE, &use_trace_fd, N_("Set SYSPROF_TRACE_FD environment for subprocess") },
     { "gjs", 0, 0, G_OPTION_ARG_NONE, &gjs, N_("Set GJS_TRACE_FD environment to trace GJS processes") },
     { "gtk", 0, 0, G_OPTION_ARG_NONE, &gtk, N_("Set GTK_TRACE_FD environment to trace a GTK application") },
+    { "rapl", 0, 0, G_OPTION_ARG_NONE, &rapl, N_("Include RAPL energy statistics") },
     { "gnome-shell", 0, 0, G_OPTION_ARG_NONE, &gnome_shell, N_("Connect to org.gnome.Shell for profiler statistics") },
     { "version", 0, 0, G_OPTION_ARG_NONE, &version, N_("Print the sysprof-cli version and exit") },
     { NULL }
@@ -330,6 +332,15 @@ main (gint   argc,
       g_object_unref (source);
     }
 
+  if (rapl)
+    {
+      source = sysprof_proxy_source_new (G_BUS_TYPE_SYSTEM,
+                                         "org.gnome.Sysprof3",
+                                         "/org/gnome/Sysprof3/RAPL");
+      sysprof_profiler_add_source (profiler, source);
+      g_object_unref (source);
+    }
+
   if (!no_network)
     {
       source = sysprof_netdev_source_new ();
